@@ -1,8 +1,8 @@
 /*****************************************************************************
  * multicat.c: netcat-equivalent for multicast
  *****************************************************************************
- * Copyright (C) 2009, 2011 VideoLAN
- * $Id: multicat.c 37 2011-12-28 21:14:36Z massiot $
+ * Copyright (C) 2009, 2011-2012, 2015 VideoLAN
+ * $Id: multicat.c 68 2015-08-01 14:31:29Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -41,15 +41,31 @@
 #include <arpa/inet.h>
 #include <pthread.h>
 #include <poll.h>
+#include <sys/ioctl.h>
+#include <syslog.h>
+
+#ifdef SIOCGSTAMPNS
+#   define HAVE_TIMESTAMPS
+#endif
+
+#ifndef POLLRDHUP
+#   define POLLRDHUP 0
+#endif
 
 #include <bitstream/ietf/rtp.h>
 #include <bitstream/mpeg/ts.h>
+#include <bitstream/mpeg/pes.h>
 
 #include "util.h"
 
+#undef DEBUG_WRITEBACK
 #define POLL_TIMEOUT 1000 /* 1 s */
 #define MAX_LATENESS INT64_C(27000000) /* 1 s */
-#define FILE_FLUSH INT64_C(27000000) /* 1 s */
+#define FILE_FLUSH INT64_C(2700000) /* 100 ms */
+#define MAX_PIDS 8192
+#define POW2_33 UINT64_C(8589934592)
+#define TS_CLOCK_MAX (POW2_33 * 27000000 / 90000)
+#define MAX_PCR_INTERVAL (27000000 / 2)
 
 /*****************************************************************************
  * Local declarations
@@ -65,29 +81,38 @@ static bool b_input_udp = false, b_output_udp = false;
 static size_t i_asked_payload_size = DEFAULT_PAYLOAD_SIZE;
 static size_t i_rtp_header_size = RTP_HEADER_SIZE;
 static uint64_t i_rotate_size = DEFAULT_ROTATE_SIZE;
+static struct udprawpkt pktheader;
+static bool b_raw_packets = false;
+static uint8_t *pi_pid_cc_table = NULL;
+/* PCR/PTS/DTS restamping */
+static uint64_t i_last_pcr_date;
+static uint64_t i_last_pcr = TS_CLOCK_MAX;
+static uint64_t i_pcr_offset;
 
-static volatile sig_atomic_t b_die = 0;
+static volatile sig_atomic_t b_die = 0, b_error = 0;
 static uint16_t i_rtp_seqnum;
 static uint64_t i_stc = 0; /* system time clock, used for date calculations */
 static uint64_t i_first_stc = 0;
 static uint64_t i_pcr = 0, i_pcr_stc = 0; /* for RTP/TS output */
-uint64_t (*pf_Date)(void) = wall_Date;
-void (*pf_Sleep)( uint64_t ) = wall_Sleep;
-ssize_t (*pf_Read)( void *p_buf, size_t i_len );
-bool (*pf_Delay)(void) = NULL;
-void (*pf_ExitRead)(void);
-ssize_t (*pf_Write)( const void *p_buf, size_t i_len );
-void (*pf_ExitWrite)(void);
+static uint64_t (*pf_Date)(void) = wall_Date;
+static void (*pf_Sleep)( uint64_t ) = wall_Sleep;
+static ssize_t (*pf_Read)( void *p_buf, size_t i_len );
+static bool (*pf_Delay)(void) = NULL;
+static void (*pf_ExitRead)(void);
+static ssize_t (*pf_Write)( const void *p_buf, size_t i_len );
+static void (*pf_ExitWrite)(void);
 
 static void usage(void)
 {
-    msg_Raw( NULL, "Usage: multicat [-i <RT priority>] [-t <ttl>] [-X] [-T <file name>] [-f] [-p <PCR PID>] [-s <chunks>] [-n <chunks>] [-k <start time>] [-d <duration>] [-a] [-r <file duration>] [-S <SSRC IP>] [-u] [-U] [-m <payload size>] [-R <RTP header size>] <input item> <output item>" );
+    msg_Raw( NULL, "Usage: multicat [-i <RT priority>] [-l <syslogtag>] [-t <ttl>] [-X] [-T <file name>] [-f] [-p <PCR PID>] [-C] [-P] [-s <chunks>] [-n <chunks>] [-k <start time>] [-d <duration>] [-a] [-r <file duration>] [-S <SSRC IP>] [-u] [-U] [-m <payload size>] [-R <RTP header size>] [-w] <input item> <output item>" );
     msg_Raw( NULL, "    item format: <file path | device path | FIFO path | directory path | network host>" );
     msg_Raw( NULL, "    host format: [<connect addr>[:<connect port>]][@[<bind addr][:<bind port>]]" );
     msg_Raw( NULL, "    -X: also pass-through all packets to stdout" );
     msg_Raw( NULL, "    -T: write an XML file with the current characteristics of transmission" );
     msg_Raw( NULL, "    -f: output packets as fast as possible" );
     msg_Raw( NULL, "    -p: overwrite or create RTP timestamps using PCR PID (MPEG-2/TS)" );
+    msg_Raw( NULL, "    -C: rewrite continuity counters to be continuous" );
+    msg_Raw( NULL, "    -P: restamp PCRs, DTSs, and PTSs" );
     msg_Raw( NULL, "    -s: skip the first N chunks of payload [deprecated]" );
     msg_Raw( NULL, "    -n: exit after playing N chunks of payload [deprecated]" );
     msg_Raw( NULL, "    -k: start at the given position (in 27 MHz units, negative = from the end)" );
@@ -99,6 +124,7 @@ static void usage(void)
     msg_Raw( NULL, "    -U: destination has no RTP header" );
     msg_Raw( NULL, "    -m: size of the payload chunk, excluding optional RTP header (default 1316)" );
     msg_Raw( NULL, "    -R: size of the optional RTP header (default 12)" );
+    msg_Raw( NULL, "    -w: send with RAW (needed for /srcaddr)" );
     exit(EXIT_FAILURE);
 }
 
@@ -107,7 +133,7 @@ static void usage(void)
  *****************************************************************************/
 static void SigHandler( int i_signal )
 {
-    b_die = 1;
+    b_die = b_error = 1;
 }
 
 /*****************************************************************************
@@ -125,13 +151,13 @@ static bool Poll(void)
     if ( i_ret < 0 )
     {
         msg_Err( NULL, "poll error (%s)", strerror(errno) );
-        b_die = 1;
+        b_die = b_error = 1;
         return false;
     }
     if ( pfd.revents & (POLLERR | POLLRDHUP | POLLHUP) )
     {
         msg_Err( NULL, "poll error" );
-        b_die = 1;
+        b_die = b_error = 1;
         return false;
     }
     if ( !i_ret ) return false;
@@ -158,7 +184,7 @@ static ssize_t tcp_Read( void *p_buf, size_t i_len )
     if ( (i_read_size = recv( i_input_fd, p_read_buffer, i_read_size, 0 )) < 0 )
     {
         msg_Err( NULL, "recv error (%s)", strerror(errno) );
-        b_die = 1;
+        b_die = b_error = 1;
         return 0;
     }
 
@@ -196,10 +222,16 @@ static ssize_t udp_Read( void *p_buf, size_t i_len )
         if ( (i_ret = recv( i_input_fd, p_buf, i_len, 0 )) < 0 )
         {
             msg_Err( NULL, "recv error (%s)", strerror(errno) );
-            b_die = 1;
+            b_die = b_error = 1;
             return 0;
         }
 
+#ifdef HAVE_TIMESTAMPS
+        struct timespec ts;
+        if ( !ioctl( i_input_fd, SIOCGSTAMPNS, &ts ) )
+            i_stc = ts.tv_sec * UINT64_C(27000000) + ts.tv_nsec * 27 / 1000;
+        else
+#endif
         i_stc = pf_Date();
     }
     else
@@ -224,16 +256,56 @@ static int udp_InitRead( const char *psz_arg, size_t i_len,
                          off_t i_nb_skipped_chunks, int64_t i_pos )
 {
     if ( i_pos || (i_input_fd = OpenSocket( psz_arg, i_ttl, DEFAULT_PORT, 0,
-                                            NULL, &b_tcp )) < 0 )
+                                            NULL, &b_tcp, NULL )) < 0 )
         return -1;
 
     i_udp_nb_skips = i_nb_skipped_chunks;
 
     pf_Read = udp_Read;
     pf_ExitRead = udp_ExitRead;
+#ifdef HAVE_TIMESTAMPS
+    if ( !b_tcp )
+        pf_Date = real_Date;
+#endif
     return 0;
 }
 
+static ssize_t raw_Write( const void *p_buf, size_t i_len )
+{
+#ifndef __APPLE__
+    ssize_t i_ret;
+    struct iovec iov[2];
+
+    #ifdef __FAVOR_BSD
+    pktheader.udph.uh_ulen
+    #else
+    pktheader.udph.len
+    #endif
+    = htons(sizeof(struct udphdr) + i_len);
+
+    iov[0].iov_base = &pktheader;
+    iov[0].iov_len = sizeof(struct udprawpkt);
+
+    iov[1].iov_base = (void *) p_buf;
+    iov[1].iov_len = i_len;
+
+    if ( (i_ret = writev( i_output_fd, iov, 2 )) < 0 )
+    {
+        if ( errno == EBADF || errno == ECONNRESET || errno == EPIPE )
+        {
+            msg_Err( NULL, "write error (%s)", strerror(errno) );
+            b_die = b_error = 1;
+        }
+        /* otherwise do not set b_die because these errors can be transient */
+        return 0;
+    }
+
+    return i_ret;
+#else
+    return -1;
+#endif
+}
+
 /* Please note that the write functions also work for TCP */
 static ssize_t udp_Write( const void *p_buf, size_t i_len )
 {
@@ -243,7 +315,7 @@ static ssize_t udp_Write( const void *p_buf, size_t i_len )
         if ( errno == EBADF || errno == ECONNRESET || errno == EPIPE )
         {
             msg_Err( NULL, "write error (%s)", strerror(errno) );
-            b_die = 1;
+            b_die = b_error = 1;
         }
         /* otherwise do not set b_die because these errors can be transient */
         return 0;
@@ -259,11 +331,20 @@ static void udp_ExitWrite(void)
 
 static int udp_InitWrite( const char *psz_arg, size_t i_len, bool b_append )
 {
-    if ( (i_output_fd = OpenSocket( psz_arg, i_ttl, 0, DEFAULT_PORT,
-                                    NULL, NULL )) < 0 )
-        return -1;
+    struct opensocket_opt opt;
 
-    pf_Write = udp_Write;
+    memset(&opt, 0, sizeof(struct opensocket_opt));
+    if (b_raw_packets) {
+        opt.p_raw_pktheader = &pktheader;
+    }
+    if ( (i_output_fd = OpenSocket( psz_arg, i_ttl, 0, DEFAULT_PORT,
+                                    NULL, NULL, &opt )) < 0 )
+        return -1;
+    if (b_raw_packets) { 
+        pf_Write = raw_Write;
+    } else {
+        pf_Write = udp_Write;
+    }
     pf_ExitWrite = udp_ExitWrite;
     return 0;
 }
@@ -288,7 +369,7 @@ static ssize_t stream_Read( void *p_buf, size_t i_len )
     if ( (i_ret = read( i_input_fd, p_buf, i_len )) < 0 )
     {
         msg_Err( NULL, "read error (%s)", strerror(errno) );
-        b_die = 1;
+        b_die = b_error = 1;
         return 0;
     }
 
@@ -328,7 +409,7 @@ retry:
         if (errno == EAGAIN || errno == EINTR)
             goto retry;
         msg_Err( NULL, "write error (%s)", strerror(errno) );
-        b_die = 1;
+        b_die = b_error = 1;
     }
     return i_ret;
 }
@@ -360,7 +441,7 @@ static ssize_t file_Read( void *p_buf, size_t i_len )
     if ( (i_ret = read( i_input_fd, p_buf, i_len )) < 0 )
     {
         msg_Err( NULL, "read error (%s)", strerror(errno) );
-        b_die = 1;
+        b_die = b_error = 1;
         return 0;
     }
     if ( i_ret == 0 )
@@ -373,7 +454,7 @@ static ssize_t file_Read( void *p_buf, size_t i_len )
     if ( fread( p_aux, 8, 1, p_input_aux ) != 1 )
     {
         msg_Warn( NULL, "premature end of aux file reached" );
-        b_die = 1;
+        b_die = b_error = 1;
         return 0;
     }
     i_stc = FromSTC( p_aux );
@@ -446,19 +527,27 @@ static ssize_t file_Write( const void *p_buf, size_t i_len )
 {
     uint8_t p_aux[8];
     ssize_t i_ret;
+#ifdef DEBUG_WRITEBACK
+    uint64_t start = pf_Date(), end;
+#endif
 
     if ( (i_ret = write( i_output_fd, p_buf, i_len )) < 0 )
     {
-        msg_Err( NULL, "couldn't write to file" );
-        b_die = 1;
+        msg_Err( NULL, "couldn't write to file (%s)", strerror(errno) );
+        b_die = b_error = 1;
         return i_ret;
     }
+#ifdef DEBUG_WRITEBACK
+    end = pf_Date();
+    if (end - start > 270000) /* 10 ms */
+        msg_Err(NULL, "too long waiting in write(%"PRId64")", (end - start) / 27000);
+#endif
 
     ToSTC( p_aux, i_stc );
     if ( fwrite( p_aux, 8, 1, p_output_aux ) != 1 )
     {
         msg_Err( NULL, "couldn't write to auxiliary file" );
-        b_die = 1;
+        b_die = b_error = 1;
     }
     if (!i_file_next_flush)
         i_file_next_flush = i_stc + FILE_FLUSH;
@@ -666,9 +755,8 @@ static void GetPCR( const uint8_t *p_buffer, size_t i_read_size )
         if ( !ts_validate( p_buffer ) )
         {
             msg_Warn( NULL, "invalid TS packet (sync=0x%x)", p_buffer[0] );
-            return;
         }
-        if ( (i_pid == i_pcr_pid || i_pcr_pid == 8192)
+        else if ( (i_pid == i_pcr_pid || i_pcr_pid == 8192)
               && ts_has_adaptation(p_buffer) && ts_get_adaptation(p_buffer)
               && tsaf_has_pcr(p_buffer) )
         {
@@ -680,13 +768,138 @@ static void GetPCR( const uint8_t *p_buffer, size_t i_read_size )
     }
 }
 
+/*****************************************************************************
+ * FixCC: fix continuity counters
+ *****************************************************************************/
+static void FixCC( uint8_t *p_buffer, size_t i_read_size )
+{
+    while ( i_read_size >= TS_SIZE )
+    {
+        uint16_t i_pid = ts_get_pid( p_buffer );
+
+        if ( !ts_validate( p_buffer ) )
+        {
+            msg_Warn( NULL, "invalid TS packet (sync=0x%x)", p_buffer[0] );
+        }
+        else
+        {
+            if ( pi_pid_cc_table[i_pid] == 0x10 )
+            {
+                msg_Dbg( NULL, "new pid entry %d", i_pid );
+                pi_pid_cc_table[i_pid] = 0;
+            }
+            else if ( ts_has_payload( p_buffer ) )
+            {
+                pi_pid_cc_table[i_pid] = (pi_pid_cc_table[i_pid] + 1) % 0x10; 
+            }
+            ts_set_cc( p_buffer, pi_pid_cc_table[i_pid] );
+        }
+        p_buffer += TS_SIZE;
+        i_read_size -= TS_SIZE;
+    }
+}
+
+/*****************************************************************************
+ * RestampPCR
+ *****************************************************************************/
+static void RestampPCR(uint8_t *p_ts)
+{
+    uint64_t i_pcr = tsaf_get_pcr(p_ts) * 300 + tsaf_get_pcrext(p_ts);
+    bool b_discontinuity = tsaf_has_discontinuity(p_ts);
+
+    if (i_last_pcr == TS_CLOCK_MAX)
+        i_last_pcr = i_pcr;
+    else {
+        /* handle 2^33 wrap-arounds */
+        uint64_t i_delta =
+            (TS_CLOCK_MAX + i_pcr -
+             (i_last_pcr % TS_CLOCK_MAX)) % TS_CLOCK_MAX;
+        if (i_delta <= MAX_PCR_INTERVAL && !b_discontinuity)
+            i_last_pcr = i_pcr;
+        else {
+            msg_Warn( NULL, "PCR discontinuity (%"PRIu64")", i_delta );
+            i_last_pcr += i_stc - i_last_pcr_date;
+            i_last_pcr %= TS_CLOCK_MAX;
+            i_pcr_offset += TS_CLOCK_MAX + i_last_pcr - i_pcr;
+            i_pcr_offset %= TS_CLOCK_MAX;
+            i_last_pcr = i_pcr;
+        }
+    }
+    i_last_pcr_date = i_stc;
+    if (!i_pcr_offset)
+        return;
+
+    i_pcr += i_pcr_offset;
+    i_pcr %= TS_CLOCK_MAX;
+    tsaf_set_pcr(p_ts, i_pcr / 300);
+    tsaf_set_pcrext(p_ts, i_pcr % 300);
+    tsaf_clear_discontinuity(p_ts);
+}
+
+/*****************************************************************************
+ * RestampTS
+ *****************************************************************************/
+static uint64_t RestampTS(uint64_t i_ts)
+{
+    i_ts += i_pcr_offset;
+    i_ts %= TS_CLOCK_MAX;
+    return i_ts;
+}
+
+/*****************************************************************************
+ * Restamp: Restamp PCRs, DTSs and PTSs
+ *****************************************************************************/
+static void Restamp( uint8_t *p_buffer, size_t i_read_size )
+{
+    while ( i_read_size >= TS_SIZE )
+    {
+        if ( !ts_validate( p_buffer ) )
+        {
+            msg_Warn( NULL, "invalid TS packet (sync=0x%x)", p_buffer[0] );
+        }
+        else
+        {
+            if (ts_has_adaptation(p_buffer) && ts_get_adaptation(p_buffer) &&
+                tsaf_has_pcr(p_buffer))
+                RestampPCR(p_buffer);
+
+            uint16_t header_size = TS_HEADER_SIZE +
+                                   (ts_has_adaptation(p_buffer) ? 1 : 0) +
+                                   ts_get_adaptation(p_buffer);
+            if (ts_get_unitstart(p_buffer) && ts_has_payload(p_buffer) &&
+                header_size + PES_HEADER_SIZE_PTS <= TS_SIZE &&
+                pes_validate(p_buffer + header_size) &&
+                pes_get_streamid(p_buffer + header_size) !=
+                    PES_STREAM_ID_PRIVATE_2 &&
+                pes_validate_header(p_buffer + header_size) &&
+                pes_has_pts(p_buffer + header_size) &&
+                pes_validate_pts(p_buffer + header_size)) {
+                pes_set_pts(p_buffer + header_size,
+                        RestampTS(pes_get_pts(p_buffer + header_size) * 300) /
+                        300);
+
+                if (header_size + PES_HEADER_SIZE_PTSDTS <= TS_SIZE &&
+                    pes_has_dts(p_buffer + header_size) &&
+                    pes_validate_dts(p_buffer + header_size))
+                    pes_set_dts(p_buffer + header_size,
+                        RestampTS(pes_get_dts(p_buffer + header_size) * 300) /
+                        300);
+            }
+        }
+        p_buffer += TS_SIZE;
+        i_read_size -= TS_SIZE;
+    }
+}
+
 /*****************************************************************************
  * Entry point
  *****************************************************************************/
 int main( int i_argc, char **pp_argv )
 {
     int i_priority = -1;
+    const char *psz_syslog_tag = NULL;
     bool b_passthrough = false;
+    bool b_restamp = false;
     int i_stc_fd = -1;
     off_t i_skip_chunks = 0, i_nb_chunks = -1;
     int64_t i_seek = 0;
@@ -699,7 +912,7 @@ int main( int i_argc, char **pp_argv )
     sigset_t set;
 
     /* Parse options */
-    while ( (c = getopt( i_argc, pp_argv, "i:t:XT:fp:s:n:k:d:ar:S:uUm:R:h" )) != -1 )
+    while ( (c = getopt( i_argc, pp_argv, "i:l:t:XT:fp:CPs:n:k:d:ar:S:uUm:R:wh" )) != -1 )
     {
         switch ( c )
         {
@@ -707,6 +920,10 @@ int main( int i_argc, char **pp_argv )
             i_priority = strtol( optarg, NULL, 0 );
             break;
 
+        case 'l':
+            psz_syslog_tag = optarg;
+            break;
+
         case 't':
             i_ttl = strtol( optarg, NULL, 0 );
             break;
@@ -730,6 +947,15 @@ int main( int i_argc, char **pp_argv )
             i_pcr_pid = strtol( optarg, NULL, 0 );
             break;
 
+        case 'C':
+            pi_pid_cc_table = malloc(MAX_PIDS * sizeof(uint8_t));
+            memset(pi_pid_cc_table, 0x10, MAX_PIDS * sizeof(uint8_t));
+            break;
+
+        case 'P':
+            b_restamp = true;
+            break;
+
         case 's':
             i_skip_chunks = strtol( optarg, NULL, 0 );
             break;
@@ -780,6 +1006,10 @@ int main( int i_argc, char **pp_argv )
             i_rtp_header_size = strtol( optarg, NULL, 0 );
             break;
 
+        case 'w':
+            b_raw_packets = true;
+            break;
+
         case 'h':
         default:
             usage();
@@ -789,6 +1019,9 @@ int main( int i_argc, char **pp_argv )
     if ( optind >= i_argc - 1 )
         usage();
 
+    if ( psz_syslog_tag != NULL )
+        msg_Openlog( psz_syslog_tag, LOG_NDELAY, LOG_USER );
+
     /* Open sockets */
     if ( udp_InitRead( pp_argv[optind], i_asked_payload_size, i_skip_chunks,
                        i_seek ) < 0 )
@@ -917,6 +1150,10 @@ int main( int i_argc, char **pp_argv )
             i_payload_size = i_read_size;
         }
 
+        /* Skip last incomplete TS packet */
+        i_read_size -= i_payload_size % TS_SIZE;
+        i_payload_size -= i_payload_size % TS_SIZE;
+
         /* Pad to get the asked payload size */
         while ( i_payload_size + TS_SIZE <= i_asked_payload_size )
         {
@@ -925,6 +1162,14 @@ int main( int i_argc, char **pp_argv )
             i_payload_size += TS_SIZE;
         }
 
+        /* Fix continuity counters */
+        if ( pi_pid_cc_table != NULL )
+            FixCC( p_payload, i_payload_size );
+
+        /* Restamp */
+        if ( b_restamp )
+            Restamp( p_payload, i_payload_size );
+
         /* Prepare header and size of output */
         if ( b_output_udp )
         {
@@ -1000,9 +1245,13 @@ dropped_packet:
             break;
     }
 
+    free(pi_pid_cc_table);
+
     pf_ExitRead();
     pf_ExitWrite();
 
-    return EXIT_SUCCESS;
-}
+    if ( psz_syslog_tag != NULL )
+        msg_Closelog();
 
+    return b_error ? EXIT_FAILURE : EXIT_SUCCESS;
+}
