@@ -1,8 +1,8 @@
 /*****************************************************************************
  * util.c: Utils for the multicat suite
  *****************************************************************************
- * Copyright (C) 2004, 2009, 2011 VideoLAN
- * $Id: util.c 38 2012-01-03 21:16:18Z massiot $
+ * Copyright (C) 2004, 2009, 2011, 2015 VideoLAN
+ * $Id: util.c 66 2015-07-17 16:03:17Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -42,9 +42,12 @@
 #include <unistd.h>
 #include <sys/mman.h>
 #include <netdb.h>
+#include <syslog.h>
 
 #include "util.h"
 
+#define DEBUG_SOCKET
+
 /*****************************************************************************
  * Local declarations
  *****************************************************************************/
@@ -53,32 +56,49 @@
 #define PSZ_TS_EXT "ts"
 
 int i_verbose = VERB_DBG;
+static int b_syslog = 0;
 
 /*****************************************************************************
- * sockaddr_t: wrapper to avoid strict-aliasing issues
+ * msg_Openlog
  *****************************************************************************/
-typedef union
+void msg_Openlog( const char *ident, int option, int facility )
 {
-    struct sockaddr_storage ss;
-    struct sockaddr so;
-    struct sockaddr_in sin;
-    struct sockaddr_in6 sin6;
-} sockaddr_t;
+    openlog(ident, option, facility);
+    b_syslog = 1;
+}
+
+/*****************************************************************************
+ * msg_Closelog
+ *****************************************************************************/
+void msg_Closelog( void )
+{
+    closelog();
+    b_syslog = 0;
+}
 
 /*****************************************************************************
  * msg_Info
  *****************************************************************************/
 void msg_Info( void *_unused, const char *psz_format, ... )
 {
-    if ( i_verbose >= VERB_INFO )
-    {
-        va_list args;
-        char psz_fmt[MAX_MSG];
-        va_start( args, psz_format );
+    if ( i_verbose < VERB_INFO )
+        return;
 
+    va_list args;
+    va_start( args, psz_format );
+
+    if ( !b_syslog )
+    {
+        char psz_fmt[MAX_MSG];
         snprintf( psz_fmt, MAX_MSG, "info: %s\n", psz_format );
         vfprintf( stderr, psz_fmt, args );
     }
+    else
+    {
+        vsyslog( LOG_INFO, psz_format, args );
+    }
+
+    va_end( args );
 }
 
 /*****************************************************************************
@@ -87,11 +107,20 @@ void msg_Info( void *_unused, const char *psz_format, ... )
 void msg_Err( void *_unused, const char *psz_format, ... )
 {
     va_list args;
-    char psz_fmt[MAX_MSG];
     va_start( args, psz_format );
 
-    snprintf( psz_fmt, MAX_MSG, "error: %s\n", psz_format );
-    vfprintf( stderr, psz_fmt, args );
+    if ( !b_syslog )
+    {
+        char psz_fmt[MAX_MSG];
+        snprintf( psz_fmt, MAX_MSG, "error: %s\n", psz_format );
+        vfprintf( stderr, psz_fmt, args );
+    }
+    else
+    {
+        vsyslog( LOG_ERR, psz_format, args );
+    }
+
+    va_end( args );
 }
 
 /*****************************************************************************
@@ -99,15 +128,24 @@ void msg_Err( void *_unused, const char *psz_format, ... )
  *****************************************************************************/
 void msg_Warn( void *_unused, const char *psz_format, ... )
 {
-    if ( i_verbose >= VERB_WARN )
-    {
-        va_list args;
-        char psz_fmt[MAX_MSG];
-        va_start( args, psz_format );
+    if ( i_verbose < VERB_WARN )
+        return;
 
+    va_list args;
+    va_start( args, psz_format );
+
+    if ( !b_syslog )
+    {
+        char psz_fmt[MAX_MSG];
         snprintf( psz_fmt, MAX_MSG, "warning: %s\n", psz_format );
         vfprintf( stderr, psz_fmt, args );
     }
+    else
+    {
+        vsyslog( LOG_WARNING, psz_format, args );
+    }
+
+    va_end( args );
 }
 
 /*****************************************************************************
@@ -115,19 +153,28 @@ void msg_Warn( void *_unused, const char *psz_format, ... )
  *****************************************************************************/
 void msg_Dbg( void *_unused, const char *psz_format, ... )
 {
-    if ( i_verbose >= VERB_DBG )
-    {
-        va_list args;
-        char psz_fmt[MAX_MSG];
-        va_start( args, psz_format );
+    if ( i_verbose < VERB_DBG )
+        return;
 
+    va_list args;
+    va_start( args, psz_format );
+
+    if ( !b_syslog )
+    {
+        char psz_fmt[MAX_MSG];
         snprintf( psz_fmt, MAX_MSG, "debug: %s\n", psz_format );
         vfprintf( stderr, psz_fmt, args );
     }
+    else
+    {
+        vsyslog( LOG_DEBUG, psz_format, args );
+    }
+
+    va_end( args );
 }
 
 /*****************************************************************************
- * msg_Raw
+ * msg_Raw: only used for usage()
  *****************************************************************************/
 void msg_Raw( void *_unused, const char *psz_format, ... )
 {
@@ -137,6 +184,7 @@ void msg_Raw( void *_unused, const char *psz_format, ... )
 
     snprintf( psz_fmt, MAX_MSG, "%s\n", psz_format );
     vfprintf( stderr, psz_fmt, args );
+    va_end( args );
 }
 
 /*****************************************************************************
@@ -148,12 +196,12 @@ static uint64_t _wall_Date( bool b_realtime )
     struct timespec ts;
 
     /* Try to use POSIX monotonic clock if available */
-    if( b_realtime || clock_gettime( CLOCK_MONOTONIC, &ts ) == EINVAL )
+    if( b_realtime || clock_gettime( CLOCK_MONOTONIC, &ts ) == -1 )
         /* Run-time fallback to real-time clock (always available) */
         (void)clock_gettime( CLOCK_REALTIME, &ts );
 
     return ((uint64_t)ts.tv_sec * (uint64_t)27000000)
-            + (uint64_t)(ts.tv_nsec * 27 / 1000);
+            + ((uint64_t)ts.tv_nsec * 27 / 1000);
 #else
     struct timeval tv_date;
 
@@ -214,6 +262,7 @@ void real_Sleep( uint64_t i_delay )
  *****************************************************************************/
 static int GetInterfaceIndex( const char *psz_name )
 {
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
     int i_fd;
     struct ifreq ifr;
 
@@ -235,6 +284,9 @@ static int GetInterfaceIndex( const char *psz_name )
     close( i_fd );
 
     return ifr.ifr_ifindex;
+#else
+    return 0;
+#endif
 }
 
 /*****************************************************************************
@@ -339,6 +391,35 @@ static struct addrinfo *ParseNodeService( char *_psz_string, char **ppsz_end,
     if ( psz_node[0] == '\0' )
         psz_node = "0.0.0.0";
 
+    if ( i_family != AF_INET6 )
+    {
+        /* Give a try to inet_aton because experience shows that getaddrinfo()
+         * fails in certain cases, like when network is down. */
+        struct in_addr addr;
+        if ( inet_aton( psz_node, &addr ) != 0 )
+        {
+            struct sockaddr_in *p_sin = malloc( sizeof(struct sockaddr_in) );
+            p_sin->sin_family = AF_INET;
+            if ( psz_port != NULL )
+                p_sin->sin_port = ntohs( atoi( psz_port ) );
+            else
+                p_sin->sin_port = 0;
+            p_sin->sin_addr = addr;
+
+            p_res = malloc( sizeof(struct addrinfo) );
+            p_res->ai_family = AF_INET;
+            p_res->ai_socktype = SOCK_DGRAM;
+            p_res->ai_protocol = 0;
+            p_res->ai_addrlen = sizeof(struct sockaddr_in);
+            p_res->ai_addr = (struct sockaddr *)p_sin;
+            p_res->ai_canonname = NULL;
+            p_res->ai_next = NULL;
+
+            free( psz_string );
+            return p_res;
+        }
+    }
+
     memset( &hint, 0, sizeof(hint) );
     hint.ai_family = i_family;
     hint.ai_socktype = SOCK_DGRAM;
@@ -355,14 +436,86 @@ static struct addrinfo *ParseNodeService( char *_psz_string, char **ppsz_end,
     return p_res;
 }
 
+/*****************************************************************************
+ * RawFillHeaders - fill ip/udp headers for RAW socket
+ *****************************************************************************/
+static void RawFillHeaders(struct udprawpkt *dgram,
+                        in_addr_t ipsrc, in_addr_t ipdst,
+                        uint16_t portsrc, uint16_t portdst,
+                        uint8_t ttl, uint8_t tos, uint16_t len)
+{
+#ifndef __APPLE__
+    struct iphdr *iph = &(dgram->iph);
+    struct udphdr *udph = &(dgram->udph);
+
+#ifdef DEBUG_SOCKET
+    char ipsrc_str[16], ipdst_str[16];
+    struct in_addr insrc, indst;
+    insrc.s_addr = ipsrc;
+    indst.s_addr = ipdst;
+    strncpy(ipsrc_str, inet_ntoa(insrc), 16);
+    strncpy(ipdst_str, inet_ntoa(indst), 16);
+    printf("Filling raw header (%p) (%s:%u -> %s:%u)\n", dgram, ipsrc_str, portsrc, ipdst_str, portdst);
+#endif
+
+    // Fill ip header
+    iph->ihl      = 5;              // ip header with no specific option
+    iph->version  = 4;
+    iph->tos      = tos;
+    iph->tot_len  = sizeof(struct udprawpkt) + len; // auto-htoned ?
+    iph->id       = htons(0);       // auto-generated if frag_off (flags) = 0 ?
+    iph->frag_off = 0;
+    iph->ttl      = ttl;
+    iph->protocol = IPPROTO_UDP;
+    iph->check    = 0;
+    iph->saddr    = ipsrc;
+    iph->daddr    = ipdst;
+
+    // Fill udp header
+    #ifdef __FAVOR_BSD
+    udph->uh_sport = htons(portsrc);
+    udph->uh_dport = htons(portdst);
+    udph->uh_ulen  = htons(sizeof(struct udphdr) + len);
+    udph->uh_sum   = 0;
+    #else
+    udph->source = htons(portsrc);
+    udph->dest   = htons(portdst);
+    udph->len    = htons(sizeof(struct udphdr) + len);
+    udph->check  = 0;
+    #endif
+
+    // Compute ip header checksum. Computed by kernel when frag_off = 0 ?
+    //iph->check = csum((unsigned short *)iph, sizeof(struct iphdr));
+#endif
+}
+
 /*****************************************************************************
  * OpenSocket: parse argv and open IPv4 & IPv6 sockets
  *****************************************************************************/
+static char *config_stropt( char *psz_string )
+{
+    char *ret, *tmp;
+    if ( !psz_string || strlen( psz_string ) == 0 )
+        return NULL;
+    ret = tmp = strdup( psz_string );
+    while (*tmp) {
+        if (*tmp == '_')
+            *tmp = ' ';
+        if (*tmp == '/') {
+            *tmp = '\0';
+            break;
+        }
+        tmp++;
+    }
+    return ret;
+}
+
 int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
-                uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp )
+                uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
+                struct opensocket_opt *p_opt)
 {
     sockaddr_t bind_addr, connect_addr;
-    int i_fd, i;
+    int i_fd = -1, i;
     char *psz_arg = strdup(_psz_arg);
     char *psz_token = psz_arg;
     char *psz_token2 = NULL;
@@ -373,6 +526,11 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     struct addrinfo *p_ai;
     int i_family;
     socklen_t i_sockaddr_len;
+    bool b_host = false;
+    bool b_raw_packets = false;
+    in_addr_t i_raw_srcaddr = INADDR_ANY; 
+    int i_raw_srcport = 0;
+    char *psz_ifname = NULL;
 
     bind_addr.ss.ss_family = AF_UNSPEC;
     connect_addr.ss.ss_family = AF_UNSPEC;
@@ -401,6 +559,7 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     /* Hosts */
     if ( psz_token[0] != '@' )
     {
+        b_host = true;
         p_ai = ParseNodeService( psz_token, &psz_token, i_connect_port,
                                  &i_connect_if_index );
         if ( p_ai == NULL )
@@ -424,6 +583,17 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
          connect_addr.ss.ss_family == AF_UNSPEC )
         return -1;
 
+#ifdef DEBUG_SOCKET
+    PrintSocket( "socket definition:", &bind_addr, &connect_addr );
+#endif
+    /* opensocket optional struct */
+    if (p_opt) {
+        if (p_opt->p_raw_pktheader) {
+            memset(p_opt->p_raw_pktheader, 0, sizeof(struct udprawpkt));
+            b_raw_packets = true;
+        }
+    }
+
     /* Weights and options */
     if ( psz_token2 )
     {
@@ -438,13 +608,34 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
                 i_bind_if_index = i_connect_if_index =
                     strtol( ARG_OPTION("ifindex="), NULL, 0 );
             else if ( IS_OPTION("ifaddr=") )
-                i_if_addr = inet_addr( ARG_OPTION("ifaddr=") );
+            {
+                char *option = config_stropt( ARG_OPTION("ifaddr=") );
+                i_if_addr = inet_addr( option );
+                free( option );
+            }
+            else if ( IS_OPTION("ifname=") )
+            {
+                psz_ifname = config_stropt( ARG_OPTION("ifname=") );
+                if (strlen(psz_ifname) >= IFNAMSIZ) {
+                    psz_ifname[IFNAMSIZ-1] = '\0';
+                }
+            }
             else if ( IS_OPTION("ttl=") )
                 i_ttl = strtol( ARG_OPTION("ttl="), NULL, 0 );
             else if ( IS_OPTION("tos=") )
                 i_tos = strtol( ARG_OPTION("tos="), NULL, 0 );
             else if ( IS_OPTION("tcp") )
                 *pb_tcp = true;
+            else if ( IS_OPTION("srcaddr=") )
+            {
+                char *option = config_stropt( ARG_OPTION("srcaddr=") );
+                i_raw_srcaddr = inet_addr( option );
+                free( option );
+            }
+            else if ( IS_OPTION("srcport=") )
+                i_raw_srcport = strtol( ARG_OPTION("srcport="), NULL, 0 );
+            else if ( IS_OPTION("fd=") )
+                i_fd = strtol( ARG_OPTION("fd="), NULL, 0 );
             else
                 msg_Warn( NULL, "unrecognized option %s", psz_token2 );
 
@@ -486,74 +677,87 @@ int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
     else i_connect_if_index = i_bind_if_index;
 
     /* Socket configuration */
-    if ( (i_fd = socket( i_family, *pb_tcp ? SOCK_STREAM : SOCK_DGRAM,
-                         0 )) < 0 )
+    if ( i_fd < 0 )
     {
-        msg_Err( NULL, "unable to open socket (%s)", strerror(errno) );
-        exit(EXIT_FAILURE);
-    }
+        if (b_raw_packets && b_host)
+        { 
+            RawFillHeaders(p_opt->p_raw_pktheader,
+                i_raw_srcaddr, connect_addr.sin.sin_addr.s_addr, i_raw_srcport,
+                ntohs(connect_addr.sin.sin_port), i_ttl, i_tos, 0);
+            i_fd = socket( AF_INET, SOCK_RAW, IPPROTO_RAW );
+        } else
+            i_fd = socket( i_family, *pb_tcp ? SOCK_STREAM : SOCK_DGRAM, 0);
 
-    i = 1;
-    if ( setsockopt( i_fd, SOL_SOCKET, SO_REUSEADDR, (void *)&i,
-                     sizeof(i) ) == -1 )
-    {
-        msg_Err( NULL, "unable to set socket (%s)", strerror(errno) );
-        exit(EXIT_FAILURE);
-    }
-
-    if ( i_family == AF_INET6 )
-    {
-        if ( i_bind_if_index
-              && setsockopt( i_fd, IPPROTO_IPV6, IPV6_MULTICAST_IF,
-                     (void *)&i_bind_if_index, sizeof(i_bind_if_index) ) < 0 )
+        if ( i_fd < 0 )
         {
-            msg_Err( NULL, "couldn't set interface index" );
-            PrintSocket( "socket definition:", &bind_addr, &connect_addr );
+            msg_Err( NULL, "unable to open socket (%s)", strerror(errno) );
             exit(EXIT_FAILURE);
         }
 
-        if ( bind_addr.ss.ss_family != AF_UNSPEC )
+        i = 1;
+        if ( setsockopt( i_fd, SOL_SOCKET, SO_REUSEADDR, (void *)&i,
+                         sizeof(i) ) == -1 )
         {
-            if ( IN6_IS_ADDR_MULTICAST( &bind_addr.sin6.sin6_addr ) )
+            msg_Err( NULL, "unable to set socket (%s)", strerror(errno) );
+            exit(EXIT_FAILURE);
+        }
+
+        if ( i_family == AF_INET6 )
+        {
+            if ( i_bind_if_index
+                  && setsockopt( i_fd, IPPROTO_IPV6, IPV6_MULTICAST_IF,
+                         (void *)&i_bind_if_index, sizeof(i_bind_if_index) ) < 0 )
             {
-                struct ipv6_mreq imr;
-                sockaddr_t bind_addr_any = bind_addr;
-                bind_addr_any.sin6.sin6_addr = in6addr_any;
+                msg_Err( NULL, "couldn't set interface index" );
+                PrintSocket( "socket definition:", &bind_addr, &connect_addr );
+                exit(EXIT_FAILURE);
+            }
 
-                if ( bind( i_fd, &bind_addr_any.so,
-                           sizeof(bind_addr_any) ) < 0 )
+            if ( bind_addr.ss.ss_family != AF_UNSPEC )
+            {
+                #ifndef __APPLE__
+                if ( IN6_IS_ADDR_MULTICAST( &bind_addr.sin6.sin6_addr ) )
                 {
-                    msg_Err( NULL, "couldn't bind" );
-                    PrintSocket( "socket definition:", &bind_addr,
-                                 &connect_addr );
-                    exit(EXIT_FAILURE);
-                }
+                    struct ipv6_mreq imr;
+                    sockaddr_t bind_addr_any = bind_addr;
+                    bind_addr_any.sin6.sin6_addr = in6addr_any;
+
+                    if ( bind( i_fd, &bind_addr_any.so,
+                               sizeof(bind_addr_any) ) < 0 )
+                    {
+                        msg_Err( NULL, "couldn't bind" );
+                        PrintSocket( "socket definition:", &bind_addr,
+                                     &connect_addr );
+                        exit(EXIT_FAILURE);
+                    }
 
-                imr.ipv6mr_multiaddr = bind_addr.sin6.sin6_addr;
-                imr.ipv6mr_interface = i_bind_if_index;
+                    imr.ipv6mr_multiaddr = bind_addr.sin6.sin6_addr;
+                    imr.ipv6mr_interface = i_bind_if_index;
 
-                /* Join Multicast group without source filter */
-                if ( setsockopt( i_fd, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP,
-                                 (char *)&imr, sizeof(struct ipv6_mreq) ) < 0 )
-                {
-                    msg_Err( NULL, "couldn't join multicast group" );
-                    PrintSocket( "socket definition:", &bind_addr,
-                                  &connect_addr );
-                    exit(EXIT_FAILURE);
+                    /* Join Multicast group without source filter */
+                    if ( setsockopt( i_fd, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP,
+                                     (char *)&imr, sizeof(struct ipv6_mreq) ) < 0 )
+                    {
+                        msg_Err( NULL, "couldn't join multicast group" );
+                        PrintSocket( "socket definition:", &bind_addr,
+                                      &connect_addr );
+                        exit(EXIT_FAILURE);
+                    }
                 }
+                else
+                #endif
+                    goto normal_bind;
             }
-            else
-                goto normal_bind;
         }
-    }
-    else if ( bind_addr.ss.ss_family != AF_UNSPEC )
-    {
-normal_bind:
-        if ( bind( i_fd, &bind_addr.so, i_sockaddr_len ) < 0 )
+        else if ( bind_addr.ss.ss_family != AF_UNSPEC )
         {
-            msg_Err( NULL, "couldn't bind" );
-            PrintSocket( "socket definition:", &bind_addr, &connect_addr );
-            exit(EXIT_FAILURE);
+normal_bind:
+            if ( bind( i_fd, &bind_addr.so, i_sockaddr_len ) < 0 )
+            {
+                msg_Err( NULL, "couldn't bind" );
+                PrintSocket( "socket definition:", &bind_addr, &connect_addr );
+                exit(EXIT_FAILURE);
+            }
         }
     }
 
@@ -568,6 +772,7 @@ normal_bind:
         if ( bind_addr.ss.ss_family == AF_INET
               && IN_MULTICAST( ntohl(bind_addr.sin.sin_addr.s_addr)) )
         {
+#ifdef IP_ADD_SOURCE_MEMBERSHIP /* unavailable on GNU Hurd */
             if ( connect_addr.ss.ss_family != AF_UNSPEC )
             {
                 /* Source-specific multicast */
@@ -580,13 +785,18 @@ normal_bind:
 
                 if ( setsockopt( i_fd, IPPROTO_IP, IP_ADD_SOURCE_MEMBERSHIP,
                             (char *)&imr, sizeof(struct ip_mreq_source) ) < 0 )
+                {
                     msg_Err( NULL, "couldn't join multicast group (%s)",
                              strerror(errno) );
                     PrintSocket( "socket definition:", &bind_addr,
                                  &connect_addr );
                     exit(EXIT_FAILURE);
+                }
             }
-            else if ( i_bind_if_index )
+            else
+#endif
+#ifndef __FreeBSD__
+            if ( i_bind_if_index )
             {
                 /* Linux-specific interface-bound multicast */
                 struct ip_mreqn imr;
@@ -596,13 +806,16 @@ normal_bind:
 
                 if ( setsockopt( i_fd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                                  (char *)&imr, sizeof(struct ip_mreqn) ) < 0 )
+                {
                     msg_Err( NULL, "couldn't join multicast group (%s)",
                              strerror(errno) );
                     PrintSocket( "socket definition:", &bind_addr,
                                  &connect_addr );
                     exit(EXIT_FAILURE);
+                }
             }
             else
+#endif
             {
                 /* Regular multicast */
                 struct ip_mreq imr;
@@ -619,6 +832,18 @@ normal_bind:
                     exit(EXIT_FAILURE);
                 }
             }
+#ifdef SO_BINDTODEVICE
+            if (psz_ifname) {
+                if ( setsockopt( i_fd, SOL_SOCKET, SO_BINDTODEVICE,
+                                 psz_ifname, strlen(psz_ifname)+1 ) < 0 ) {
+                    msg_Err( NULL, "couldn't bind to device %s (%s)",
+                             psz_ifname, strerror(errno) );
+                    exit(EXIT_FAILURE);
+                }
+                free(psz_ifname);
+                psz_ifname = NULL;
+            }
+#endif
         }
     }
 
