@@ -1,8 +1,8 @@
 /*****************************************************************************
  * util.h: Utils for the multicat suite
  *****************************************************************************
- * Copyright (C) 2009, 2011 VideoLAN
- * $Id: util.h 36 2011-11-17 21:04:25Z massiot $
+ * Copyright (C) 2009, 2011, 2014-2015 VideoLAN
+ * $Id: util.h 64 2015-07-15 20:47:39Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -21,7 +21,14 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
+#include <netinet/udp.h>
+#include <netinet/ip.h>
+
+#ifdef __APPLE__
+#define POLLRDHUP 0
+#else
 #define HAVE_CLOCK_NANOSLEEP
+#endif
 
 #define DEFAULT_PORT 1234
 #define DEFAULT_PAYLOAD_SIZE 1316
@@ -33,9 +40,42 @@
 #define VERB_INFO 2
 #define VERB_WARN 1
 
+/*****************************************************************************
+ * sockaddr_t: wrapper to avoid strict-aliasing issues
+ *****************************************************************************/
+typedef union
+{
+    struct sockaddr_storage ss;
+    struct sockaddr so;
+    struct sockaddr_in sin;
+    struct sockaddr_in6 sin6;
+} sockaddr_t;
+
+/*****************************************************************************
+ * Raw udp packet structure with flexible-array payload
+ *****************************************************************************/
+struct udprawpkt {
+#ifndef __APPLE__
+    struct  iphdr iph;
+    struct  udphdr udph;
+    uint8_t payload[];
+#endif
+} __attribute__((packed));
+
+
+/*****************************************************************************
+ * OpenSocket options
+ *****************************************************************************/
+ struct opensocket_opt {
+    struct udprawpkt *p_raw_pktheader;
+ };
+
+
 /*****************************************************************************
  * Prototypes
  *****************************************************************************/
+void msg_Openlog( const char *ident, int option, int facility );
+void msg_Closelog( void );
 void msg_Info( void *_unused, const char *psz_format, ... );
 void msg_Err( void *_unused, const char *psz_format, ... );
 void msg_Warn( void *_unused, const char *psz_format, ... );
@@ -46,7 +86,8 @@ void wall_Sleep( uint64_t i_delay );
 uint64_t real_Date( void );
 void real_Sleep( uint64_t i_delay );
 int OpenSocket( const char *_psz_arg, int i_ttl, uint16_t i_bind_port,
-                uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp );
+                uint16_t i_connect_port, unsigned int *pi_weight, bool *pb_tcp,
+                struct opensocket_opt *p_opt);
 mode_t StatFile(const char *psz_arg);
 int OpenFile( const char *psz_arg, bool b_read, bool b_append );
 char *GetAuxFile( const char *psz_arg, size_t i_payload_size );
